/**************************************************************************************************
    Filename:       sample_app.c
    Revised:        $Date: 2008-04-10 19:47:13 -0700 (Thu, 10 Apr 2008) $
    Revision:       $Revision: 16806 $

    Description:

    This file contains the main functionality of the Sample Application to the Z-Accel slave.


    Copyright 2006-2007 Texas Instruments Incorporated. All rights reserved.

    IMPORTANT: Your use of this Software is limited to those specific rights
    granted under the terms of a software license agreement between the user
    who downloaded the software, his/her employer (which must be your employer)
    and Texas Instruments Incorporated (the "License").  You may not use this
    Software unless you agree to abide by the terms of the License. The License
    limits your use, and you acknowledge, that the Software may not be modified,
    copied or distributed unless embedded on a Texas Instruments microcontroller
    or used solely and exclusively in conjunction with a Texas Instruments radio
    frequency transceiver, which is integrated into your product.  Other than for
    the foregoing purpose, you may not use, reproduce, copy, prepare derivative
    works of, modify, distribute, perform, display or sell this Software and/or
    its documentation for any purpose.

    YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
    PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
    INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
    NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
    TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
    NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
    LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
    INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
    OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
    OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
    (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

    Should you have any questions regarding your right to use this Software,
    contact Texas Instruments Incorporated at www.TI.com.
**************************************************************************************************/

/* ------------------------------------------------------------------------------------------------
 *                                          Includes
 * ------------------------------------------------------------------------------------------------
 */

#include "zaccel.h"
#include "hal_board.h"
#include "mt.h"
#include "sample_app.h"
#include "sapi.h"

/* ------------------------------------------------------------------------------------------------
 *                                           Constants
 * ------------------------------------------------------------------------------------------------
 */

#define ZASA_PROFILE_ID                   0x0F10
#define ZASA_PROFILE_ID_LSB               0x10
#define ZASA_PROFILE_ID_MSB               0x0F

#define SRCE_REPORT_ID_LSB                1
#define SRCE_REPORT_ID_MSB                0
#define SRCE_REPORT_ID                    0x0001
#define SRCE_REPORT_SZ                    2
#define SRCE_REPORT_TEMP                  0
#define SRCE_REPORT_BUSV                  1

#define SINK_ENDPOINT_ID                  2
#define SINK_DEVICE_ID_LSB                4
#define SINK_DEVICE_ID_MSB                0
#define SINK_DEVICE_VERSION               1
#define SINK_LATENCY                      0
#define SINK_CLUSTER_IN_CNT               1
#define SINK_CLUSTER_OUT_CNT              0

#define SRCE_ENDPOINT_ID                  1
#define SRCE_DEVICE_ID_LSB                3
#define SRCE_DEVICE_ID_MSB                0
#define SRCE_DEVICE_VERSION               1
#define SRCE_LATENCY                      0
#define SRCE_CLUSTER_IN_CNT               0
#define SRCE_CLUSTER_OUT_CNT              1

static const uint8 sinkEP[] = {
  SINK_ENDPOINT_ID,
  ZASA_PROFILE_ID_LSB,
  ZASA_PROFILE_ID_MSB,
  SINK_DEVICE_ID_LSB,
  SINK_DEVICE_ID_MSB,
  SINK_DEVICE_VERSION,
  SINK_LATENCY,
  SINK_CLUSTER_IN_CNT,
  SRCE_REPORT_ID_LSB,
  SRCE_REPORT_ID_MSB,
  SINK_CLUSTER_OUT_CNT
};

static const uint8 srceEP[] = {
  SRCE_ENDPOINT_ID,
  ZASA_PROFILE_ID_LSB,
  ZASA_PROFILE_ID_MSB,
  SRCE_DEVICE_ID_LSB,
  SRCE_DEVICE_ID_MSB,
  SRCE_DEVICE_VERSION,
  SRCE_LATENCY,
  SRCE_CLUSTER_IN_CNT,
  SRCE_CLUSTER_OUT_CNT,
  SRCE_REPORT_ID_LSB,
  SRCE_REPORT_ID_MSB
};

#define APP_BTN_INTERVAL                  2      // Count button presses in a 2-second interval.

#define APP_BLINK_LEDS                    TRUE   // Turn LED status display on or off.
#define APP_BLINK_INTERVAL                1      // Blink at 1-Hz.
#define APP_BLINK_ON_TIME                 1      // LED on for 1 msec is visible.

/* This timeout for attempting to join must be greatly lengthened as channels are added to the
 * ZACCEL_NV_CHANLIST in sample_app.cfg.
 * It is a trial and error empirical process to find the right setting according to the number of
 * channels to be scanned and the channel access availability on each of those channels.
 */
#define APP_JOIN_TIME                     6      //  6 secs to attempt to join.
#define APP_JOIN_WAIT                     30     // 30 secs to sleep before re-attempt to join.
#define APP_BIND_TIME                     10     // 10 secs to attempt to bind.
#define APP_BIND_WAIT                     30     // 30 secs to sleep before re-attempt to bind.

#define APP_PMT_BIND                      0xFF   // Allow binding indefinitely.
#define APP_DENY_BIND                     0x00   // Dis-allow binding indefinitely.
#define APP_PMT_JOIN                      0xFF   // Allow joining indefinitely.
#define APP_DENY_JOIN                     0x00   // Dis-allow joining indefinitely.

#define APP_REPORT_INTERVAL               10     // 10 secs between OTA reports.
#define APP_REPORT_RETRY                  2      //  2 secs between retries when no ack received.
#define APP_RETRY_CNT                     3      // Number of retry attempts.
#define APP_DATA_CNF                      TRUE   // Request APP-layer data confirmation.

#if APP_DATA_CNF
#define APP_POLL_INTERVAL                (APP_REPORT_RETRY * 1000 / 2)
#define APP_RESPONSE_POLL                 100
#else
#define APP_POLL_INTERVAL                 0
#define APP_RESPONSE_POLL                 0
#endif

#define APP_STAT_LED                      0
#define APP_DATA_LED                      1

/* ------------------------------------------------------------------------------------------------
 *                                           Typedefs
 * ------------------------------------------------------------------------------------------------
 */

typedef enum {
  appIniting,
  appWaiting,
  appJoining,
  appJoinWaiting,
  appBinding,
  appBindWaiting,
  appRunning
} AppState;  // Valid appState values.

/* ------------------------------------------------------------------------------------------------
 *                                           Macros
 * ------------------------------------------------------------------------------------------------
 */

// All devices besides the Coordinator-Sink are Sources.
#define APP_SRCE  ((appFlags & appSinkingF) == 0)

/* ------------------------------------------------------------------------------------------------
 *                                           Global Variables
 * ------------------------------------------------------------------------------------------------
 */

uint8 appFlags;

/* ------------------------------------------------------------------------------------------------
 *                                           Local Variables
 * ------------------------------------------------------------------------------------------------
 */

//__no_init volatile int tempOffset @ 0x10F4;  // Temperature offset set at production.

static AppState appState;
#if APP_DATA_CNF
static uint8 appMsgRtry;
#endif
static uint8 appMsgHandle;
static uint8 srceReport[SRCE_REPORT_SZ];

/* ------------------------------------------------------------------------------------------------
 *                                           Local Functions
 * ------------------------------------------------------------------------------------------------
 */

// Triggered by HAL flags.
static void appExec(void);
static void appBtnPress(void);

// Helper functions for appExec().
static void appJoinFail(void);

// Helper functions for appBtnPress().
static void appStartRequest(uint8 rtrType);
static void appToggleJoin(void);

// Triggered by ZACCEL flags or ZACCEL response.
static void appReset(void);
static void appStart(void);
#if APP_DATA_CNF
static void appDataCnf(void);
#else
#define appDataCnf()
#endif
static void appSinkData(void);
static void appSrceBind(void);
static void appSrceData(void);
static void appSetPoll(void);
#if APP_BLINK_LEDS
static void appLedBlink(uint8 led);
#else
#define appLedBlink(led)
#endif

/**************************************************************************************************
 * @fn          appInit
 *
 * @brief       This function is the host application initialization.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
void appInit(void)
{
#if APP_BLINK_LEDS
  // Setup the LED blink at 1-Hz.
  halTimerSet(HAL_IDX_TIMER_LED, APP_BLINK_INTERVAL, HAL_TIMER_AUTO);
#endif

  HAL_ENABLE_INTERRUPTS();
}

/**************************************************************************************************
 * @fn          appExecHal
 *
 * @brief       This function is the ZASA executive for HAL events.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      TRUE if a HAL event was processed; FALSE otherwise.
 **************************************************************************************************
 */
uint16 appExecHal(void)
{
  uint16 event = HAL_EVT_NONE;
  halIntState_t s;

  if (halEventFlags & HAL_EVT_TIMER_LED)
  {
    event = HAL_EVT_TIMER_LED;
    appLedBlink(APP_STAT_LED);
  }
  else if (halEventFlags & HAL_EVT_TIMER_APP)
  {
    event = HAL_EVT_TIMER_APP;
    appExec();
  }
  else if (halEventFlags & HAL_EVT_TIMER_BTN)
  {
    event = HAL_EVT_TIMER_BTN;
    appBtnPress();
  }
  else if (halEventFlags & HAL_EVT_BTN_PRESS)
  {
    event = HAL_EVT_BTN_PRESS;

    halTimerSet(HAL_IDX_TIMER_BTN, APP_BTN_INTERVAL, 0);

    // Immediately turn of LEDs when user starts a join process.
    if (appState == appWaiting)
    {
      // Stop the LED blink during joining.
      halTimerSet(HAL_IDX_TIMER_LED, 0, 0);
      HAL_TURN_OFF_GRN();
      HAL_TURN_OFF_RED();
    }
  }
  else if (halEventFlags & HAL_EVT_ADC)
  {
    event = HAL_EVT_ADC;
    appSrceData();
  }
#if HOST_MT
  else if (halEventFlags & HAL_EVT_MT_RX_RDY)
  {
    if (mtRx())
    {
      event = HAL_EVT_MT_RX_RDY;
    }
  }
#endif
  else if (halEventFlags & HAL_EVT_NO_8MHz)
  {
    event = HAL_EVT_NO_8MHz;
  }

  /* Since HAL event flags are set at the interrupt level, they must only be cleared within
   * a critical section.
   */
  if (event != ZACCEL_EVT_NONE)
  {
    HAL_ENTER_CRITICAL_SECTION(s);
    event = halEventFlags & event;
    halEventFlags ^= event;
    HAL_EXIT_CRITICAL_SECTION(s);
    return TRUE;
  }

  return FALSE;
}

/**************************************************************************************************
 * @fn          appExecHost
 *
 * @brief       This function is the ZASA executive for ZACCEL events.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      TRUE if a ZACCEL event was processed; FALSE otherwise.
 **************************************************************************************************
 */
uint16 appExecHost(void)
{
  uint16 event = ZACCEL_EVT_NONE;

  zaccelPoll();

  if (zaccelEvtFlags & ZACCEL_SYS_RESET_IND)
  {
    event = ZACCEL_SYS_RESET_IND;
    appReset();
  }
  else if (zaccelEvtFlags & ZACCEL_START_CNF)
  {
    event = ZACCEL_START_CNF;

    if (appState == appJoining)
    {
      appStart();
    }
  }
  else if (zaccelEvtFlags & ZACCEL_BIND_CNF)
  {
    event = ZACCEL_BIND_CNF;

    if (APP_SRCE)
    {
      // Setup an auto-repeating timer to send periodic status reports OTA.
      halTimerSet(HAL_IDX_TIMER_APP, APP_REPORT_INTERVAL, HAL_TIMER_AUTO);
      // But send a report immediately from here to reduce latency of 1st report from a new node.
      appSrceData();
      appState = appRunning;
    }
  }
  else if (zaccelEvtFlags & ZACCEL_SEND_DATA_CNF)
  {
    event = ZACCEL_SEND_DATA_CNF;

    if (APP_SRCE)
    {
      appDataCnf();
    }
  }
  else if (zaccelEvtFlags & ZACCEL_RCV_DATA_IND)
  {
    event = ZACCEL_RCV_DATA_IND;
    appSinkData();
  }

  if (event != ZACCEL_EVT_NONE)
  {
    event = zaccelEvtFlags & event;
    zaccelEvtFlags ^= event;
    return TRUE;
  }

  return FALSE;
}

/**************************************************************************************************
 * @fn          appExec
 *
 * @brief       This function is the ZASA executive run by a periodic timer event.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appExec(void)
{
  switch (appState)
  {
  case appIniting:
  case appWaiting:
    // Not expected in this state, so just stop the timer in case it is auto-repeating.
    halTimerSet(HAL_IDX_TIMER_APP, 0, 0);
    break;

  case appJoining:
    // Attempting to join did not succeed within the allowed APP_NEW_JOIN_TIME.
    appJoinFail();
    break;

  case appJoinWaiting:
    // Set rtrType=False since only an RFD uses this state (to sleep and retry joining.)
    appStartRequest(0);
    break;

  case appBinding:
    halTimerSet(HAL_IDX_TIMER_APP, APP_BIND_WAIT, 0);
    appState = appBindWaiting;
    break;

  case appBindWaiting:
    appSrceBind();
    break;

  case appRunning:
    if (APP_SRCE)
    {
      if (appFlags & appSendingF)
      {
        appDataCnf();
      }
      else if (appFlags & appSetPollF)
      {
        appSetPoll();
      }
      else
      {
        appSrceData();
      }
    }
    break;
  }
}

/**************************************************************************************************
 * @fn          appBtnPress
 *
 * @brief       This function acts on a button press.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appBtnPress(void)
{
  switch (appState)
  {
  case appIniting:
    // Do not act on a button press in this state which should be only momentary after powerup.
    break;

  case appWaiting:
    appStartRequest((halSwCnt == 1));
    break;

  case appJoining:
  case appJoinWaiting:
  case appBinding:
  case appBindWaiting:
    // Do not act on a button press in these states.
    break;

  case appRunning:
    if (ZACCEL_NWK_RTR)
    {
      appToggleJoin();
    }
    break;
  }

  halSwCnt = 0;
}

/**************************************************************************************************
 * @fn          appJoinFail
 *
 * @brief       This function executes the specified behaviour when a join attempt fails.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appJoinFail(void)
{
  uint8 val;

  /* MT_SAPI_START_CNF can be missed by the race condition between re-registering the Endpoint
   * with the SAPI after the device does a reset to restore/auto-join and the join success.
   * This check catches the situation when a join succeeds before this host can even register
   * the Endpoint.
   */
  zb_GetDeviceInfo(ZB_INFO_DEV_STATE);
  if (ZACCEL_NWK_CONN)
  {
    appStart();
    return;
  }

  /* If attempting to join as an RFD/End Device fails, the only way to stop it is to reset ZACCEL.
   * Sleep for SRCE_EXEC_TIME and re-try joining.
   */
  if (appFlags & appLowPwrF)
  {
    val = ZCD_STARTOPT_RESTORE_STATE;
    zb_WriteConfiguration(ZCD_NV_STARTUP_OPTION, 1, &val);
    appState = appJoinWaiting;
    halTimerSet(HAL_IDX_TIMER_APP, APP_JOIN_WAIT, 0);
  }
  else  // If attempting to join as an FFD/Router fails, start a new network as a Coordinator.
  {
    val = ZG_DEVICETYPE_COORDINATOR;
    zb_WriteConfiguration(ZCD_NV_LOGICAL_TYPE, 1, &val);
    appFlags |= appSinkingF;
  }
  zb_SystemReset();
}

/**************************************************************************************************
 * @fn          appStartRequest
 *
 * @brief       This function acts on a button press.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appStartRequest(uint8 rtrType)
{
  uint8 tmp;

  if (rtrType)
  {
    tmp = ZG_DEVICETYPE_ROUTER;
  }
  else
  {
    tmp = ZG_DEVICETYPE_ENDDEVICE;
    appFlags |= (appLowPwrF | appSetPollF);
  }
  zb_WriteConfiguration(ZCD_NV_LOGICAL_TYPE, 1, &tmp);
  zb_StartRequest();
  appState = appJoining;

  /* The ZACCEL will try to join indefinitely, so the host is setting a timer in order to take
   * action if the join attempt does not succeed within a reasonable amount of time.
   */
  halTimerSet(HAL_IDX_TIMER_APP, APP_JOIN_TIME, 0);
}

/**************************************************************************************************
 * @fn          appToggleJoin
 *
 * @brief       This function acts on a request to toggle the permit join state of the ZACCEL.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appToggleJoin(void)
{
  // Toggle the "permit joining" of the ZACCEL and set the LED behavior accordingly.
  if (appFlags & appPermittingF)
  {
    appFlags &= ~appPermittingF;
    zb_PermitJoiningRequest(zaccelNwkAddr, 0x00);

    // A Coordinator-Sink device starts blinking the Red LED when not permitting join.
    if (ZACCEL_NWK_COOR)
    {
      HAL_TURN_OFF_RED();
    }
    // A Router-Source device starts blinking the Green LED when not permitting join.
    else
    {
      HAL_TURN_OFF_GRN();
    }

    // Setup the LED blink at 1-Hz.
    halTimerSet(HAL_IDX_TIMER_LED, APP_BLINK_INTERVAL, HAL_TIMER_AUTO);
  }
  else
  {
    // Stop the LED blink.
    halTimerSet(HAL_IDX_TIMER_LED, 0, 0);

    // A Router-Source device sets the Red LED solid on when permitting join.
    if (ZACCEL_NWK_COOR)
    {
      HAL_TURN_ON_RED();
    }
    // A Router-Source device sets the Green LED solid on when permitting join.
    else
    {
      HAL_TURN_ON_GRN();
    }

    appFlags |= appPermittingF;
    zb_PermitJoiningRequest(zaccelNwkAddr, 0xFF);
  }
}

/**************************************************************************************************
 * @fn          appReset
 *
 * @brief       This function is the host action on a ZACCEL reset.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appReset(void)
{
  // No previously received indication flag can be valid after the ZACCEL resets.
  zaccelIndFlags = 0;

  /* No ZigBee Endpoints (not even the Simple Descriptor) are not stored in the ZACCEL NV.
   * Therefore, the host must re-register anytime that the ZACCEL resets.
   */
  if (APP_SRCE)
  {
    zb_SapiAppRegister(srceEP);
  }
  else
  {
    zb_SapiAppRegister(sinkEP);
  }

  switch (appState)
  {
  case appIniting:
    {
      uint8 val = ZCD_STARTOPT_CLEAR_CONFIG;

      // Reset Network NV items.
      zb_WriteConfiguration(ZCD_NV_STARTUP_OPTION, 1, &val);
      zb_SystemReset();

      appState = appWaiting;
    }
    break;

  case appWaiting:
    // The last step of the appIniting above was to reset the ZACCEL.
    {
      uint32 val32;

      // Configure the Host Application-specific defaults from sample_app.cfg into the ZACCEL.
      val32 = ZACCEL_NV_PANID;
      zb_WriteConfiguration(ZCD_NV_PANID, 2, &val32);
      val32 = ZACCEL_NV_CHANLIST;
      zb_WriteConfiguration(ZCD_NV_CHANLIST, 4, &val32);
    }
    break;

  case appJoining:
    /* This host application has configured the ZACCEL as a ZigBee device and then reset it so that
     * the ZACCEL can re-configure its RAM accordingly and start an auto-join process from powerup.
     * No action other than registering the Endpoint, already done above, is required.
     * The host will receive a join indication from the ZACCEL or the joining timer will expire.
     */
  case appJoinWaiting:
    /* This host application has configured the ZACCEL to stop attempting to join so that low power
     * can be entered while waiting to re-try joinin.
     */
    break;

  case appBinding:
  case appBindWaiting:
  case appRunning:
    break;
  }
}

/**************************************************************************************************
 * @fn          appStart
 *
 * @brief       This function is the host application registration with the ZACCEL SAPI.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appStart(void)
{
  // And now allow the Z-Accel slave to NV restore and auto re-start on resets.
  uint8 val = ZCD_STARTOPT_AUTO_START;
  zb_WriteConfiguration(ZCD_NV_STARTUP_OPTION, 1, &val);

  zb_GetDeviceInfo(ZB_INFO_DEV_STATE);
  zb_GetDeviceInfo(ZB_INFO_SHORT_ADDR);

  if (ZACCEL_NWK_RTR)
  {
    appFlags &= ~appPermittingF;
    appToggleJoin();  // This will set permit joining allowed and set LEDs accordingly.
    if (!APP_SRCE)
    {
      zb_AllowBind(APP_PMT_BIND);
      appState = appRunning;
    }
  }
#if APP_BLINK_LEDS
  else
  {
    // RFD/End Device type blinks the Green LED.
    halTimerSet(HAL_IDX_TIMER_LED, APP_BLINK_INTERVAL, HAL_TIMER_AUTO);
  }
#endif

  if (APP_SRCE)
  {
    zb_AllowBind(APP_DENY_BIND);
    appSrceBind();
  }
}

#if APP_DATA_CNF
/**************************************************************************************************
 * @fn          appDataCnf
 *
 * @brief       This function is the host application action upon receiving a message confirmation.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appDataCnf(void)
{
  if (zaccelIndFlags & ZACCEL_SEND_SUCCESS)
  {
    zaccelIndFlags &= ~ZACCEL_SEND_SUCCESS;
    appFlags &= ~appSendingF;
    appLedBlink(APP_DATA_LED);

    if (appFlags & appLowPwrF)
    {
      /* The RFD had to set this timer to the shorter APP_REPORT_RETRY interval to wake from sleep
       * and retry sending the report if there was no confirmation.
       * Now RFD sets timer to wake from sleep to send the next report.
       * The FFD just leaves this timer auto-repeating at the APP_REPORT_INTERVAL.
       */
      halTimerSet(HAL_IDX_TIMER_APP, APP_REPORT_INTERVAL, HAL_TIMER_AUTO);
    }
  }
  else if ((appMsgRtry == 0) || (--appMsgRtry == 0))
  {
    appFlags &= ~appSendingF;
    appSrceBind();  // When a message confirmation fails, attempt to re-bind.
  }
  else
  {
    // Do not increment the message handle since this is the same message, only a re-try.
    zb_SendDataRequest (ZB_BINDING_ADDR, SRCE_REPORT_ID, appMsgHandle,
                        AF_ACK_REQUEST, AF_DEFAULT_RADIUS, SRCE_REPORT_SZ, srceReport);
  }
}
#endif

/**************************************************************************************************
 * @fn          appSinkData
 *
 * @brief       This function is the host application to process received data.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appSinkData(void)
{
  appLedBlink(APP_DATA_LED);
}

/**************************************************************************************************
 * @fn          appSrceBind
 *
 * @brief       This function is the host application to request an Endpoint binding.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appSrceBind(void)
{
  /* IEEE address of the device to establish the binding with. Set the destIEEE to NULL
   * in order to bind with any other device that is in the Allow Binding Mode.
   */
  uint8 destIEEE[Z_EXTADDR_LEN] = { 0, 0, 0, 0, 0, 0, 0, 0 };
  zb_BindDevice(TRUE, SRCE_REPORT_ID, destIEEE);
  halTimerSet(HAL_IDX_TIMER_APP, APP_BIND_TIME, 0);
  appState = appBinding;
}

/**************************************************************************************************
 * @fn          appSrceData
 *
 * @brief       This function is the host application to process received data.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appSrceData(void)
{
  uint16 tmp;

  switch (appFlags & (appTempF | appBusVF))
  {
  case 0:  // Idle - not reading the ADC.
    // Setup to sample air temperature.
    appFlags |= appTempF;
    halReadTemp();
    break;

  case appTempF:  // Measuring air temperature.
    appFlags &= ~appTempF;

    // oC = ((A10/1024)*1500mV)-986mV)*1/3.55mV = A10*423/1024 - 278
    srceReport[SRCE_REPORT_TEMP] = ((halAdcVal - 673) * 423) / 1024;
    /* TBD: not enough code space: if (tempOffset != 0xFFFF)
    {
      srceReport[SRCE_REPORT_TEMP] += tempOffset / 10;
    }*/

    // Setup to sample bus voltage.
    appFlags |= appBusVF;
    halReadBusV();
    break;

  case appBusVF:  // Measuring bus voltage.
    appFlags &= ~appBusVF;

    /* halAdcVal contains measurement of AVcc/2
     * halAdcVal is in range 0 to 1023 indicating voltage from 0 to 1.5V
     * voltage = (halAdcVal*2*1.5)/1023 volts
     * we will multiply by this by 10 to allow units of 0.1 volts
     */
    tmp = (halAdcVal * 30 + 512) / 1024;  // Convert to units of 0.1V
    srceReport[SRCE_REPORT_BUSV] = (uint8)tmp;

#if APP_DATA_CNF
    if (APP_SRCE)
    {
      // Increment the message handle so that the next message is unique.
      zb_SendDataRequest (ZB_BINDING_ADDR, SRCE_REPORT_ID, appMsgHandle++,
                          AF_ACK_REQUEST, AF_DEFAULT_RADIUS, SRCE_REPORT_SZ, srceReport);

      /* An RFD needs a timer to wake from sleep and poll the ZACCEL for the ZACCEL_SEND_DATA_CNF.
       * An FFD will always be awake and ready to poll for it.
       */
      if (appFlags & appLowPwrF)
      {
        halTimerSet(HAL_IDX_TIMER_APP, APP_REPORT_RETRY, HAL_TIMER_AUTO);
      }

      appMsgRtry = APP_RETRY_CNT;
      appFlags |= appSendingF;
    }
#else
    zb_SendDataRequest (ZB_BINDING_ADDR, SRCE_REPORT_ID, appMsgHandle++,
                        0, AF_DEFAULT_RADIUS, SRCE_REPORT_SZ, srceReport);
    appLedBlink(APP_DATA_LED);
#endif
    break;
  }
}

/**************************************************************************************************
 * @fn          appSetPoll
 *
 * @brief       This function is the host application to set the desired poll rates.
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appSetPoll(void)
{
  uint16 val;

  // Setup for the desired long-term poll rates.
  val = APP_POLL_INTERVAL;
  zb_WriteConfiguration(ZCD_NV_POLL_RATE, 2, &val);
  val = APP_RESPONSE_POLL;
  zb_WriteConfiguration(ZCD_NV_RESPONSE_POLL_RATE, 2, &val);
  zb_SystemReset();
  appFlags &= ~appSetPollF;
}

#if APP_BLINK_LEDS
/**************************************************************************************************
 * @fn          appLedBlink
 *
 * @brief       Blink the LED specified.
 *
 * input parameters
 *
 * @param       led - Which LED to control: Status or Data.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 **************************************************************************************************
 */
static void appLedBlink(uint8 led)
{
  switch (led)
  {
  case APP_STAT_LED:
    if (ZACCEL_NWK_CONN)
    {
      if (ZACCEL_NWK_COOR)
      {
        HAL_TURN_ON_RED();
        halDelay(APP_BLINK_ON_TIME, TRUE);
        HAL_TURN_OFF_RED();
      }
      else
      {
        HAL_TURN_ON_GRN();
        halDelay(APP_BLINK_ON_TIME, TRUE);
        HAL_TURN_OFF_GRN();
      }
    }
    else
    {
      HAL_TURN_ON_GRN();
      HAL_TURN_ON_RED();
      halDelay(APP_BLINK_ON_TIME, TRUE);
      HAL_TURN_OFF_GRN();
      HAL_TURN_OFF_RED();
    }
    break;

  case APP_DATA_LED:
    if (ZACCEL_NWK_COOR)
    {
      HAL_TURN_ON_GRN();
      halDelay(APP_BLINK_ON_TIME, TRUE);
      HAL_TURN_OFF_GRN();
    }
    else
    {
      HAL_TURN_ON_RED();
      halDelay(APP_BLINK_ON_TIME, TRUE);
      HAL_TURN_OFF_RED();
    }
    break;

  default:
    break;
  }
}
#endif
